/******************************************** IMPORTS *********************************************/
import { RealAny } from './types-iso';
/******************************************** MATCHING ********************************************/
/**
 * [Curried]
 * Returns true if array matchVals contains [valToFind]
 * Uses simple JSON.stringify for array and object comparison
 * Sane behaviour for matching against null, undefined, NaN, etc. (e.g. NaN
 * matched against an array with NaN returns true)
 *
 * @param {Array<any>} matchVals Array to check for item matching valToFind
 * @param {any} valToFind Value to search for in matchVals
 *
 * @return {boolean} true if valToFind is found in matchVals
 */
export declare const matchAny: (matchVals: any[]) => (valToFind: any) => boolean;
/**
 * Determine if array [arr] contains [value]
 * @param {Array} arr Array to check for the given value
 * @param {string} value Value to search for in the array
 * @return {boolean} true if arr contains val
 */
export declare const contains: (arr: string | any[], value: any) => boolean;
export { contains as includes };
/*********************************** ARRAY & COLLECTION HELPERS ***********************************/
/**
 * Return first item in an array
 * @param {T[]} arr Array to return first item from
 */
export declare function first<T>(arr: T[]): T;
/**
 * Return first character in a string
 * @param {T[]} str String to return first character from
 */
export declare function first(str: string): string;
/**
 * Return 2nd item in an array
 * @param {T[]} arr Array to return 2nd item from
 */
export declare function second<T>(arr: T[]): T;
/**
 * Return 2nd character in a string
 * @param {T[]} str String to return 2nd character from
 */
export declare function second<T>(str: string): string;
/**
 * Return third item in an array
 */
export declare function third<T>(arr: T[]): T;
/**
 * Return third character in a string
 */
export declare function third<T>(str: string): string;
/**
 * Return last item in an array
 */
export declare function last<T>(arr: T[]): T;
/**
 * Return last character in a string
 */
export declare function last(str: string): string;
/**
 * Return second last item in an array
 */
export declare function secondLast<T>(arr: T[]): T;
/**
 * Return second last character in a string
 */
export declare function secondLast(str: string): string;
/**
 * Return third last item in an array
 */
export declare function thirdLast<T>(arr: T[]): T;
/**
 * Return third last character in a string
 */
export declare function thirdLast(str: string): string;
/**
 * Return first 2 items in an array
 */
export declare function first2<T>(arr: T[]): T[];
/**
 * Return first 2 characters in a string
 */
export declare function first2<T>(str: string): string;
/**
 * Return first 3 items in an array
 */
export declare function first3<T>(arr: T[]): T[];
/**
 * Return first 3 characters in a string
 */
export declare function first3<T>(str: string): string;
/**
 * Return last 2 items in an array
 */
export declare function last2<T>(arr: T[]): T[];
/**
 * Return last 2 characters in a string
 */
export declare function last2<T>(str: string): string;
/**
 * Return last 3 items in an array
 */
export declare function last3<T>(arr: T[]): T[];
/**
 * Return the 1st [num] number of items in string [str]
 * @param {string} str String to return items from
 * @return 1st N items in string, or full string if more items than it contains
 *         are requested
 */
export declare function firstN(str: string, num: number): string;
/**
 * Return the 1st [num] number of items in array [arr]
 * @param {string} arr Array to return items from
 * @return 1st N items in an array, or full array if more items than array
 *         contains are requested
 */
export declare function firstN<T>(arr: T[], num: number): T[];
/** Return last N items in an array */
export declare function lastN(str: string, n: number): string;
export declare function lastN<T>(arr: T[], n: number): T[];
/**
 * Create array of [len] number of repeats of given [fillVal], or undefined
 * if [fillVal] not given
 * @param {number} len Length of array to create
 * @param {RealAny} fillVal Item to repeat 'len' number of times {OPT}
 * @return {Array<void|typeof fillVal>} Array w 'len' # of fillVal arg (or
 *                                      undefined) repeats
 */
export declare const arrayN: <T>(len: number, fillVal?: T) => T[];
/************************ EXCLUDE ITEMS FROM START OR END OF ARRAY/STRING *************************/
/**
 * Exclude first item from string or array
 * @param {Array<any>|string} arrOrStr Array/string to exclude first item from
 * @return {Array<any>} Array with first item excluded
 */
export declare function withoutFirst<T>(str: string): string;
export declare function withoutFirst<T>(arr: T[]): T[];
/**
 * Exclude last item from string or array
 * @param {Array<any>|string} arrOrStr Array/string to exclude last item from
 * @return {Array<any>} Array with last item excluded
 */
export declare function withoutLast<T>(str: string): string;
export declare function withoutLast<T>(arr: T[]): T[];
/**
 * Exclude first 2 items from string or array
 * @param {Array<any>|string} arrOrStr Array/string to exclude first 2 items from
 * @return {Array<any>} Array with first 2 items excluded
 */
export declare function withoutFirst2<T>(str: string): string;
export declare function withoutFirst2<T>(arr: T[]): T[];
/**
 * Exclude first 3 items from string or array
 * @param {Array<any>|string} arrOrStr Array/string to exclude first 3 items from
 * @return {Array<any>} Array with first 3 items excluded
 */
export declare function withoutFirst3<T>(str: string): string;
export declare function withoutFirst3<T>(arr: T[]): T[];
/**
 * Exclude last 2 items from string or array
 * @param {Array<any>|string} arrOrStr Array/string to exclude last 2 items from
 * @return {Array<any>} Array with last 2 items excluded
 */
export declare function withoutLast2<T>(str: string): string;
export declare function withoutLast2<T>(arr: T[]): T[];
/**
 * Exclude last 3 items from string or array
 * @param {Array<any>|string} arrOrStr Array/string to exclude last 3 items from
 * @return {Array<any>} Array with last 3 items excluded
 */
export declare function withoutLast3<T>(str: string): string;
export declare function withoutLast3<T>(arr: T[]): T[];
/**
 * Exclude given number of items from end of string or array
 * @param {Array<any>|string} arrOrStr Array/string to exclude last N items from
 * @return {Array<any>} Array with last N items excluded
 */
export declare function withoutLastN<T>(str: string, numToRm: number): string;
export declare function withoutLastN<T>(arr: T[], numToRm: number): T[];
/**
 * Exclude given number of items from beginning of string or array
 * @param {Array<any>|string} arrOrStr Array or string to exclude first N items from
 * @return {Array<any>} Array with first N items excluded
 */
export declare function withoutFirstN<T>(str: string, numToRm: number): string;
export declare function withoutFirstN<T>(arr: T[], numToRm: number): T[];
/**
 * [Non-mutative]
 * [PERFORMANCE-INTENSIVE]
 *
 * Return new array with all items in arr2OrItem removed from array1; or if
 * array2 is not an array, remove matching item from array1
 *
 * @param {any[]} arr1 Array to remove items from
 * @param {any[]|any} arr2OrItem Remove all items in this array, or remove item if not an array
 * @return {any[]} arr1 with all items in arr2OrItem (or the item itself) removed
 */
export declare function removeMatches(arr1: RealAny[], arr2: any): RealAny[];
export declare function removeMatches(arr1: RealAny[], arr2: RealAny[]): RealAny[];
export declare type _FalsyType = 'allFalsy' | 'nullUndef' | 'keep0' | 'keepStr';
declare type _NotFunctionNotArray = object | Object | number | string | boolean | undefined | null | void;
declare type NonFunction = _NotFunctionNotArray | any[];
/**
 * [MUTATIVE]
 * Mutatively removes all items in array where predicate returned true
 * Return new array, or array of deleted elements if returnRemovals is true
 *
 * Example: arrayRemove([-5, 0, 3, 3, 3, 8, 12, 15], (num => num > 10), true);
 *          // => [12, 15]
 *
 * @param {Array} haystack Array to remove items from
 * @param {Function} predicate Remove item for each case where this returns true
 * @param {boolean} returnRemovals If true, return removed elements instead of
 *                                 new array
 * @return {Array} haystack with all predicate matches removed, or array of all
 *                 removed elements if returnRemovals is true
 */
export declare function arrayRemove<T = RealAny>(haystack: T[], predicate: (item: T) => boolean, returnRemovals?: boolean): T[];
/**
 * [MUTATIVE]
 * Mutatively removes all matches of given value from array
 * Return new array, or array of deleted elements if returnRemovals is true
 *
 * Example: arrayRemove(['a', 'b', 'c', 'd', 'b'], 'b');
 *          // => ['a', 'c', 'd']
 *
 * @param {Array} haystack Array to remove items from
 * @param {any} needle Item to remove from array (remove ALL matches)
 * @param {boolean} returnRemovals If true, return removed elements instead of
 *                                 new array
 * @return {Array} array from haystack property with all "needle"s removed, or
 *                 if returnRemovals is true, array of all removed elements
 */
export declare function arrayRemove<T = NonFunction>(haystack: T[], needle: T, returnRemovals?: boolean): T[];
/**
 * Remove falsy values from given array [arr]
 * By default removes all falsy val types, but 2nd param can set it to only rm
 * certain falsy types ['allFalsy', 'keep0', 'keepStr', 'nullUndef']
 *
 * @param {Array} arr Array containing any values of any type
 * @param {string} falsyTypes: `allFalsy`  [DEFAULT] Remove all falsy values
 *                             `nullUndef` Remove only null & undefined values
 *                             `keep0`     Remove all falsy values except 0
 *                             `keepStr`   Remove all falsy values except ``
 * @return {Array} arr Duplicate of initial array, with falsy values of set
 *                     types removed {Default: remove all falsy values}
 */
export declare const rmAllFalsy: <T = any>(arr: T[], falsyType?: _FalsyType) => T[];
export { rmAllFalsy as compact };
export { rmAllFalsy as rmFalsyVals };
/**
 * Add item to array if item not already present in array
 *
 * @param {Array} arr Array to potentially add item to
 * @param {any} newItem Item to potentially add to array (if array doesn't
 *                      already contain it)
 * @return {Array} Initially given array, with item potentially added
 */
export declare function pushIfUniq<T = any>(arr: T[], newItem: T): T[];
export { pushIfUniq as pushIfNew };
export { pushIfUniq as pushUniq };
/**
 * Split large multiline string into array where each line is an item
 * Also removes blank lines
 *
 * @param {String} str Multiline string to split into array, where each line is
 *                     an array item (it splits on `\n` characters)
 * @param {Object} opts::
 *                 @param {boolean} preserveEmptyLines If true, remove all blank lines
 *                                                     Off by default
 * @return {Array<string>} Array where each item is a line from the input
 *                         string, with falsy values removed
 */
export declare const splitLines: (str: string, opts?: {
    preserveEmptyLines: boolean;
}) => string[];
/**
 * Count number of occurrences of value [needle] in array [haystack]
 *
 * @param {any} needle Item/character to search for
 * @param {any[]} haystack Array/string to search for the item/character
 * @return {Map<any, number>} Map of each item in the array vs its number of
 *                            occurences
 */
export declare function countOccurrences<T>(needle: T, haystack: T[]): number;
/**
 * Count number of occurrences of character [char] in string [haystack]
 *
 * @param {string} needle Item/character to search for
 * @param {string} haystack Array/string to search for the item/character
 * @return {Map<string, number>} Map of each item in the array vs its number of
 *                            occurences
 */
export declare function countOccurrences<T>(needle: string, haystack: string): number;
export { countOccurrences as count };
export { countOccurrences as countAll };
export { countOccurrences as countItems };
export { countOccurrences as countArrayItems };
/**
 * Remove duplicate characters from given string [str]
 * @param {string|Array} coll String to remove duplicates from
 * @return {string} String with no duplicate characters (unique characters only)
 */
export declare function removeDuplicates(str: string): string;
/**
 * Remove duplicate values from given array [coll]
 * @param {Array} coll Array to remove duplicates from
 * @return {Array} Array with no duplicates (unique values only)
 */
export declare function removeDuplicates<T = any>(coll: T[]): T[];
export { removeDuplicates as uniq };
export { removeDuplicates as uniqVals };
export { removeDuplicates as unique };
export { removeDuplicates as uniqueVals };
export { removeDuplicates as removeDuplicateVals };
export { removeDuplicates as removeDuplicateItems };
/**
 * Namespace for certain "reversed" operations
 */
export declare const without: {
    last: typeof withoutLast;
    last2: typeof withoutLast2;
    last3: typeof withoutLast3;
    lastN: typeof withoutLastN;
    first: typeof withoutFirst;
    first2: typeof withoutFirst2;
    first3: typeof withoutFirst3;
    firstN: typeof withoutFirstN;
};
/******************************************* COLLECTION *******************************************/
/**
 * Get a random value from an array, or return undefined if array is empty
 *
 * @param {Array} coll Array to get random value from
 * @return {any|undefined} Item randomly selected from given array
 */
export declare function sample<T = any>(coll: T[]): T | undefined;
/**
 * Return random entry ([key, value]) from given object/collection, or undefined
 * if it has no keys
 *
 * @param {Object} coll Object to get random value from
 * @return {[string, any]|undefined} Randomly selected [key, value] array from object
 */
export declare function sample<T = any>(coll: Record<string, T>): [string, T] | undefined;
/**
 * Return random character from given string, or undefined if it's an empty string
 * @param {string} coll String to get random character from
 * @return {string|undefined} Randomly selected character from string
 */
export declare function sample<T = any>(coll: string): string | undefined;
/**
 * Return random pair from given Map, or return [] if it's an empty Map
 * @param {Map<K, V>} coll Map to get random pair from
 * @return {[K, V]} Randomly selected pair from string, or [] if map is empty
 */
export declare function sample<K = any, V = any>(coll: Map<K, V>): [K, V] | undefined[];
/**
 * Return random item from given Set, or return undefined if it's an empty Set
 * @param {Set<K>} coll Set to get random item from
 * @return {K} Randomly selected item from string, or undefined if set is empty
 */
export declare function sample<K = any>(coll: Set<K>): K | undefined;
/**
 * Deeply flatten an array ([arr])
 * Example: flatten([1, [2, [3, 4]], 5]); // => [1, 2, 3, 4, 5]
 *
 * @param {Array} arr Array (or set of nested arrays) to flatten
 * @return {Array} Flattened array
 */
export declare const flatten: <T = any>(arr: T[]) => T[];
export { flatten as smoosh };
/***************************************** BARREL EXPORT ******************************************/
export { isArray } from './types-iso';
