/**
 * Character input stream.
 * Allows you to iterate through and read a string one character at a time.
 *
 * @return {Object} Keys:
 *             peek: Show next value without removing it from the stream.
 *             next:  Show the next value and discards it from the stream.
 *             isEOL: Returns true if stream is at end of line.
 *             isEOF: Returns true if stream is at end.
 *             throw: Throw input stream errors. }
 */
export declare class CharInputStream {
    input: string;
    /** Current character */
    chr: string;
    /** Current position */
    pos: number;
    /** Current column (position in current line) */
    col: number;
    /** Current line */
    line: number;
    /** @param {string} Input string to iterate through. */
    constructor(input: string);
    /** Return the next value without removing it from the stream. */
    peek: () => string;
    /**
     * Peek ahead multiple [numToPeek] chars.
     */
    peekMulti: (numToPeek: number) => string;
    /** Return the next value and discards it from the stream. */
    next: () => string;
    /** Throw error with given message, also displaying the current stream position. */
    throw: (msg: string) => never;
    /** Return true if at end of file (no more vals in stream), otherwise false */
    isEOF: () => boolean;
    /** Return true if at end of line (i.e. if given char is '\n') */
    isEOL: (ch?: string) => boolean;
    isWhitespace: (ch?: string) => boolean;
    /** Handle end of line (shift column & line) */
    private eol;
}
export { CharInputStream as CharStream };
