"use strict";
// TODO Unit test CharInputStream
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Character input stream.
 * Allows you to iterate through and read a string one character at a time.
 *
 * @return {Object} Keys:
 *             peek: Show next value without removing it from the stream.
 *             next:  Show the next value and discards it from the stream.
 *             isEOL: Returns true if stream is at end of line.
 *             isEOF: Returns true if stream is at end.
 *             throw: Throw input stream errors. }
 */
var CharInputStream = /** @class */ (function () {
    /** @param {string} Input string to iterate through. */
    function CharInputStream(input) {
        var _this = this;
        this.input = input;
        /** Current position */
        this.pos = 0;
        /** Current column (position in current line) */
        this.col = 0;
        /** Current line */
        this.line = 0;
        /*-------------------------- ITERATION METHODS ---------------------------*/
        /** Return the next value without removing it from the stream. */
        this.peek = function () { return _this.input.charAt(_this.pos); };
        /**
         * Peek ahead multiple [numToPeek] chars.
         */
        this.peekMulti = function (numToPeek) {
            var chars = '';
            for (var idx = 0; idx < numToPeek; idx++) {
                chars += _this.input.charAt(_this.pos + idx);
            }
            return chars;
        };
        /** Return the next value and discards it from the stream. */
        this.next = function () {
            // Get chr, store in object, shift pos
            _this.chr = _this.input.charAt(_this.pos++);
            // Shift row & col based on whether at eol
            if (_this.isEOL())
                _this.eol();
            else
                _this.col++;
            return _this.chr;
        };
        /*---------------------------- ERROR HANDLING ----------------------------*/
        /** Throw error with given message, also displaying the current stream position. */
        this.throw = function (msg) {
            throw new Error(msg + " (Position " + _this.pos + ": line " + _this.line + ", col " + _this.col + ")");
        };
        /*---------------------------- CHARACTER INFO ----------------------------*/
        /** Return true if at end of file (no more vals in stream), otherwise false */
        this.isEOF = function () { return _this.peek() === ''; };
        /** Return true if at end of line (i.e. if given char is '\n') */
        this.isEOL = function (ch) { return (ch || _this.chr) === '\n'; };
        /* Return true if current or given char is a whitespace char (including newline) */
        this.isWhitespace = function (ch) { return '\t\ns '.indexOf(ch || _this.chr) >= 0; };
        /*--------------------------- PRIVATE HELPERS ----------------------------*/
        /** Handle end of line (shift column & line) */
        this.eol = function () {
            _this.col = 0;
            _this.line++;
        };
    }
    return CharInputStream;
}());
exports.CharInputStream = CharInputStream;
exports.CharStream = CharInputStream;
//# sourceMappingURL=stream.js.map