"use strict";
/// <reference path="../../node_modules/@types/mocha/index.d.ts" />
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var node_1 = require("../../node");
/******************************* IMPORT FUNCTION MODULE FOR TESTING *******************************/
var node_2 = require("../../node");
var browser_1 = require("../../browser");
var functionModule = __importStar(require("../../src/function"));
var shared_1 = require("../../shared");
/********************************************* TESTS **********************************************/
describe("function sub-module", function () {
    node_1.expectNonEmptyObjectExists(shared_1.func, 'function (from shared/base export)');
    node_1.expectNonEmptyObjectExists(shared_1.m_.function, 'function (from m_ top-level namespace)');
    node_1.expectNonEmptyObjectExists(functionModule, 'function (import all from function.ts file)');
    node_1.expectNonEmptyObjectExists(node_2.func, 'function (from Node export)');
    node_1.expectNonEmptyObjectExists(browser_1.func, 'function (from Browser export)');
    it("exists", function () {
        chai_1.expect(shared_1.func).to.exist;
        chai_1.expect(shared_1.m_.function).to.exist;
    });
    describe("getFnAsArr function", function () {
        node_1.expectFunctionExists(shared_1.getFnAsArr);
        it("Displays the source code of a function in an array, with 1 item per line", function () {
            function exampleFunction() {
                console.log('I am an example');
            }
            var exampleFunctionFnArr = shared_1.func.getFnAsArr(exampleFunction);
            chai_1.expect(exampleFunctionFnArr).to.be.length(3);
            chai_1.expect(exampleFunctionFnArr).to.be.an('array');
            chai_1.expect(exampleFunctionFnArr[0]).to.match(/function exampleFunction\(\)/);
            chai_1.expect(exampleFunctionFnArr[1]).to.match(/console\.log\('I am an example'\);/);
            chai_1.expect(exampleFunctionFnArr[2]).to.match(/\}/);
        });
    });
    describe("switchExpr function", function () {
        node_1.expectFunctionExists(shared_1.switchExpr);
        it("Returns the 2nd arg if the 1st arg is truthy", function () {
            chai_1.expect(shared_1.switchExpr(true, 'val1', 'elseVal')).to.eql('val1');
            chai_1.expect(shared_1.switchExpr(true, 'val1', true, 'arg2', true, 'arg3', 'elseVal')).to.eql('val1');
            chai_1.expect(shared_1.switchExpr('cond1', 'val1', 'cond2', 'val2', 'cond3', 'val3', 'elseVal')).to.eql('val1');
        });
        it("Returns the 3rd arg if the 1st arg is falsy and only 3 args were given", function () {
            chai_1.expect(shared_1.switchExpr(false, 'val1', 'elseVal')).to.eql('elseVal');
        });
        it("Returns the last arg if all other 'condition' args are falsy", function () {
            chai_1.expect(shared_1.switchExpr(false, 'val1', undefined, 'val2', 'elseVal')).to.eql('elseVal');
            chai_1.expect(shared_1.switchExpr(0, 'v1', null, 'v2', NaN, 'v3', 'elseVal')).to.eql('elseVal');
            chai_1.expect(shared_1.switchExpr('', 'v1', false, 'v2', null, 'v3', 'elseVal')).to.eql('elseVal');
        });
        it("Allows any odd number of conditions", function () {
            chai_1.expect(shared_1.switchExpr('' + '', 'val1', undefined, 'val2', 0, 'val3', null, 'val4', NaN, 'val5', '', 'val6', false, 'val7', 0 + 0, 'val8', 'elseVal')).to.eql('elseVal');
            var nan = '' / 0;
            chai_1.expect(shared_1.switchExpr('' + '', 'val1', undefined, 'val2', 0, 'val3', null, 'val4', NaN, 'val5', '', 'val6', false, 'val7', 0 + 0, 'val8', 0 - 0, 'val9', 1 - 1, 'val10', nan, 'val11', 0 * 0, 'val12', 0 / 0, 'val13', 10099 * 0, 'val14', 'elseVal')).to.eql('elseVal');
            chai_1.expect(shared_1.switchExpr('' + '', 'val1', undefined, 'val2', 0, 'val3', null, 'val4', NaN, 'val5', '', 'val6', false, 'val7', 0 + 0, 'val8', 0 - 0, 'val9', 1 - 1, 'val10', true, 'val11', 0 * 0, 'val12', 0 / 0, 'val13', 10099 * 0, 'val14', 'elseVal')).to.eql('val11');
        });
        it("allows an even number of conditions as long as at least one condition returns truthy", function () {
            chai_1.expect(shared_1.switchExpr(true, 'v1', false, 'v2')).to.eql('v1');
            chai_1.expect(shared_1.switchExpr(false, 'v1', true, 'v2', 'c3', 'v3')).to.eql('v2');
            chai_1.expect(shared_1.switchExpr(false, 'v1', null, 'v2', 0, 'v3', 'c4', 'v4')).to.eql('v4');
        });
        it("throws if given an even number of conditions and no conditions are truthy", function () {
            chai_1.expect(function () { return shared_1.switchExpr(0, 'v1', false, 'v2'); }).to.throw(Error);
            chai_1.expect(function () { return shared_1.switchExpr(null, 'v1', false, 'v2', '', 'v3'); }).to.throw(Error);
            chai_1.expect(function () { return shared_1.switchExpr(false, 'v1', null, 'v2', '', 'v3', 0, 'v4'); }).to.throw(Error);
        });
    });
    describe("loopN", function () {
        it("runs given function requested # of times, & returns array holding all return values.", function () {
            chai_1.expect(shared_1.loopN(2, function () { return 'return_value'; })).to.eql(['return_value', 'return_value']);
            var i = 0;
            chai_1.expect(shared_1.loopN(3, function () { return ++i; })).to.eql([1, 2, 3]);
            chai_1.expect(i).to.eql(3);
            var j = 0;
            chai_1.expect(shared_1.loopN(15, function () { return j++; })).to.eql([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]);
            chai_1.expect(j).to.eql(15);
        });
    });
    describe("loop2", function () {
        it("runs given function twice, & returns array holding both return values.", function () {
            chai_1.expect(shared_1.loop2(function () { return 'ret_val'; })).to.eql(['ret_val', 'ret_val']);
            var j = 0;
            chai_1.expect(shared_1.loop2(function () { return j++; })).to.eql([0, 1]);
            chai_1.expect(j).to.eql(2);
        });
    });
    describe("loop3", function () {
        it("runs given function 3X, & returns array holding all 3 return values.", function () {
            chai_1.expect(shared_1.loop3(function () { return 'ret_val'; })).to.eql(['ret_val', 'ret_val', 'ret_val']);
            var j = 0;
            chai_1.expect(shared_1.loop3(function () { return j++; })).to.eql([0, 1, 2]);
            chai_1.expect(j).to.eql(3);
        });
    });
    describe("loop4", function () {
        it("runs given function 4X, & returns array holding all 4 return values.", function () {
            chai_1.expect(shared_1.loop4(function () { return 'ret_val'; })).to.eql(['ret_val', 'ret_val', 'ret_val', 'ret_val']);
            var j = 0;
            chai_1.expect(shared_1.loop4(function () { return j++; })).to.eql([0, 1, 2, 3]);
            chai_1.expect(j).to.eql(4);
        });
    });
    describe("loop5", function () {
        it("runs given function 5X, & returns array holding all 5 return values.", function () {
            chai_1.expect(shared_1.loop5(function () { return 'r_val'; })).to.eql(['r_val', 'r_val', 'r_val', 'r_val', 'r_val']);
            var j = 0;
            chai_1.expect(shared_1.loop5(function () { return j++; })).to.eql([0, 1, 2, 3, 4]);
            chai_1.expect(j).to.eql(5);
        });
    });
    describe("throttle", function () {
        it("runs given function immediately if immediate param = true (default)", function () {
            var val = false;
            var throttledFn = shared_1.throttle(function () { return (val = true); }, 200);
            throttledFn();
            chai_1.expect(val).to.be.true;
            var val2 = false;
            var throttledFn2 = shared_1.throttle(function () { return (val2 = true); }, 200, true);
            throttledFn2();
            chai_1.expect(val2).to.be.true;
        });
        it("doesn't run given function immediately if immediate param = false", function () {
            var val = false;
            var throttledFn = shared_1.throttle(function () { return (val = true); }, 200, false);
            throttledFn();
            chai_1.expect(val).to.be.false;
        });
        it("doesn't run function 2X if request to run occurs before [wait]ms elapses", function () {
            var val = 0;
            var throttledFn = shared_1.throttle(function () { return val++; }, 200, true);
            throttledFn();
            throttledFn();
            throttledFn();
            chai_1.expect(val).to.eql(1);
        });
        it("runs function again if request to run occurs after [wait]ms elapses", function () {
            var val = 0;
            var throttledFn = shared_1.throttle(function () { return val++; }, 50, true);
            throttledFn();
            setTimeout(function () { return throttledFn(); }, 100);
            throttledFn();
            throttledFn();
            setTimeout(function () { return chai_1.expect(val).to.eql(2); }, 150);
        });
        it("requires the correct arguments for the created 'throttled' function", function () {
            var arr = [];
            var throttledFn = shared_1.throttle(function (str) { return arr.push(str); }, 50, true);
            throttledFn('First');
            setTimeout(function () { return throttledFn('Second'); }, 100);
            throttledFn("WON'T BE INCLUDED");
            throttledFn("ALSO WON'T BE INCLUDED");
            setTimeout(function () { return chai_1.expect(arr).to.eql(['First', 'Second']); }, 150);
        });
        it("transfers data keys from given function to throttled function", function () {
            var arr = [];
            var fn = Object.assign(function () { return arr.push(fn.dataVal); }, { dataVal: 'VALUE' });
            var throttledFn = shared_1.throttle(fn, 50, true);
            chai_1.expect(throttledFn.dataVal).to.eql('VALUE');
            throttledFn();
            throttledFn();
            setTimeout(function () { return throttledFn(); }, 100);
            setTimeout(function () { return chai_1.expect(arr).to.eql(['VALUE', 'VALUE']); }, 150);
        });
    });
    describe("debounce", function () {
        it("runs given function only after [wait] ms have passed, not immediately", function (done) {
            var val = false;
            var debouncedFn = shared_1.debounce(function () { return (val = true); }, 200);
            debouncedFn();
            chai_1.expect(val).to.be.false;
            setTimeout(function () {
                chai_1.expect(val).to.be.true;
                done();
            }, 250);
        });
        it("runs given function immediately if true given as 3rd arg", function () {
            var val = false;
            var debouncedFn2 = shared_1.debounce(function () { return (val = true); }, 200, true);
            debouncedFn2();
            chai_1.expect(val).to.be.true;
        });
        it("doesn't run given function immediately if immediate param = false", function (done) {
            var val = false;
            var debouncedFn = shared_1.debounce(function () { return (val = true); }, 200);
            debouncedFn();
            chai_1.expect(val).to.be.false;
            setTimeout(function () {
                chai_1.expect(val).to.be.true;
                done();
            }, 250);
        });
        it("runs function 1X even if multiple requests received to run it before before [wait]ms elapses", function (done) {
            var val = 0;
            var debouncedFn = shared_1.debounce(function () { return val++; }, 200, true);
            debouncedFn();
            debouncedFn();
            debouncedFn();
            setTimeout(function () {
                chai_1.expect(val).to.eql(1);
                done();
            }, 250);
        });
        it("runs function again if request to run occurs after [wait]ms elapses", function (done) {
            var val = 0;
            var debouncedFn = shared_1.debounce(function () { return val++; }, 50, true);
            debouncedFn();
            setTimeout(function () { return debouncedFn(); }, 100);
            debouncedFn();
            debouncedFn();
            setTimeout(function () {
                chai_1.expect(val).to.eql(2);
                done();
            }, 150);
        });
        it("requires the correct arguments for the created 'debounced' function", function (done) {
            var arr = [];
            var debouncedFn = shared_1.debounce(function (str) { return arr.push(str); }, 50, true);
            debouncedFn('First'); // Include after initial wait (50ms)
            debouncedFn("WON'T BE INCLUDED");
            setTimeout(function () { return debouncedFn('Second'); }, 100);
            debouncedFn("ALSO WON'T BE INCLUDED");
            debouncedFn("ALSO ALSO WON'T BE INCLUDED");
            setTimeout(function () {
                chai_1.expect(arr).to.eql(['First', 'Second']);
                done();
            }, 150);
        });
        it("transfers data keys from [cb] to debounced function", function (done) {
            var arr = [];
            var fn = Object.assign(function () { return arr.push(fn.dataVal); }, { dataVal: 'VALUE' });
            var debouncedFn = shared_1.debounce(fn, 50, true);
            chai_1.expect(debouncedFn.dataVal).to.eql("VALUE");
            debouncedFn();
            debouncedFn();
            setTimeout(function () { return debouncedFn(); }, 100);
            setTimeout(function () {
                chai_1.expect(arr).to.eql(['VALUE', 'VALUE']);
                done();
            }, 170);
        });
    });
    describe('getArgNames', function () {
        it("returns empty list given arrow function with no params", function () {
            var fn = function () { return console.log('output'); };
            chai_1.expect(shared_1.getArgNames(fn)).to.eql([]);
        });
        it("gets list of arguments from given arrow function with params", function () {
            var fn = function (arg1, arg2) { return console.log('output'); };
            chai_1.expect(shared_1.getArgNames(fn)).to.eql(['arg1', 'arg2']);
        });
        it("returns empty list given classic named function with no params", function () {
            function fn() {
                console.log('output');
            }
            chai_1.expect(shared_1.getArgNames(fn)).to.eql([]);
        });
        it("gets list of arguments from given classic function with params", function () {
            function fn(arg1, arg2) {
                console.log('output');
            }
            chai_1.expect(shared_1.getArgNames(fn)).to.eql(['arg1', 'arg2']);
        });
        it("returns empty list given classic lambda with no params", function () {
            var fn = function () {
                console.log('output');
            };
            chai_1.expect(shared_1.getArgNames(fn)).to.eql([]);
        });
        it("gets list of arguments from given classic lambda with params", function () {
            var fn = function (arg1, arg2) {
                console.log('output');
            };
            chai_1.expect(shared_1.getArgNames(fn)).to.eql(['arg1', 'arg2']);
        });
    });
    describe("runAll", function () {
        it("returns empty array given an empty array", function () {
            chai_1.expect(shared_1.runAll([])).to.eql([]);
        });
        it("returns return val of function in an array, if given array w/ 1 function", function () {
            chai_1.expect(shared_1.runAll([function () { return 'result'; }])).to.eql(['result']);
        });
        it("returns array of return vals from all functions in an array", function () {
            chai_1.expect(shared_1.runAll([function () { return 'result 1'; }, function () { return 'result 2'; }, function () { return 'result 3'; }])).to.eql([
                'result 1',
                'result 2',
                'result 3',
            ]);
        });
        it("Passes args past 1st arg to all functions in array (as arguments)", function () {
            // prettier-ignore
            chai_1.expect(shared_1.runAll([
                function (name, age) { return "Hello " + age + "-year-old " + name + "!"; },
                function (name, age) { return "How are you doing, " + age + "-year-old " + name + "?"; },
            ], 'Meeka', 7)).to.eql([
                'Hello 7-year-old Meeka!',
                'How are you doing, 7-year-old Meeka?'
            ]);
        });
    });
});
//# sourceMappingURL=function.spec.js.map