"use strict";
/// <reference path="../../node_modules/@types/mocha/index.d.ts" />
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**************************************** TESTING MODULES *****************************************/
var chai_1 = require("chai");
var node_1 = require("../../node");
/******************************** IMPORT SEARCH MODULE FOR TESTING ********************************/
var shared_1 = require("../../shared");
var search_1 = require("../../src/search");
var node_2 = require("../../node");
var browser_1 = require("../../browser");
var searchModule = __importStar(require("../../src/search"));
var falseWhen = "should return false when";
var trueWhen = "should return true when";
/********************************************* TESTS **********************************************/
describe("search sub-module", function () {
    node_1.expectNonEmptyObjectExists(shared_1.search, 'search (from shared/base export)');
    node_1.expectNonEmptyObjectExists(shared_1.m_.search, 'search (from m_ top-level namespace)');
    node_1.expectNonEmptyObjectExists(searchModule, 'search (import all from search.ts file)');
    node_1.expectNonEmptyObjectExists(node_2.search, 'search (from Node export)');
    node_1.expectNonEmptyObjectExists(browser_1.search, 'search (from Browser export)');
    // Existence of several functions exported from string module via search module
    node_1.expectFunctionExists(search_1.escapeRegExp);
    node_1.expectFunctionExists(search_1.matches);
    node_1.expectFunctionExists(search_1.matchesIgnoreCase);
    node_1.expectFunctionExists(search_1.replaceAll);
    describe("fuzzySearch", function () {
        it("exists", function () {
            chai_1.expect(search_1.fuzzySearch).to.exist;
        });
        it(trueWhen + " needle and haystack are the same", function () {
            chai_1.expect(search_1.fuzzySearch("asdf", "asdf")).to.eql(true);
        });
        it(trueWhen + " haystack contains needle", function () {
            chai_1.expect(search_1.fuzzySearch("asdf", "12345asdfqwerty")).to.eql(true);
        });
        it("should be true when haystack has needle w 2 letters flipped (i.e. error-tolerant)", function () {
            chai_1.expect(search_1.fuzzySearch("asdf", "12345adsfqwerty")).to.eql(true);
        });
        it(trueWhen + " needle is in haystack, but spread out with other letters between " +
            "letters of needle (going forwards in haystack)", function () {
            chai_1.expect(search_1.fuzzySearch("asdf", "1a23s45d67f89")).to.eql(true);
        });
        it(falseWhen + " needle isn't in haystack", function () {
            chai_1.expect(search_1.fuzzySearch("asdf", "123456")).to.eql(false);
            chai_1.expect(search_1.fuzzySearch("asdf", "")).to.eql(false);
        });
        it(trueWhen + " needle is an empty string and haystack isn't", function () {
            chai_1.expect(search_1.fuzzySearch("", "123456")).to.eql(true);
            chai_1.expect(search_1.fuzzySearch("", "123456")).to.eql(true);
        });
        it(falseWhen + " an earlier needle char is >= 2 chars before a later needle char in " +
            "haystack, where there are no duplicate chars", function () {
            chai_1.expect(search_1.fuzzySearch("asdf", "s12adf3456")).to.eql(false);
            chai_1.expect(search_1.fuzzySearch("meeka", "meea12345k")).to.eql(false);
        });
        it(trueWhen + " earlier needle char is >= 2 chars before later needle char in haystack, " +
            "but the earlier needle char is a duplicate of a later needle char", function () {
            chai_1.expect(search_1.fuzzySearch("meeka", "0e123meka456789")).to.eql(true);
        });
    });
});
//# sourceMappingURL=search.spec.js.map